@@ -8,7 +8,7 @@ Date::Reformat - Rearrange date strings
 
     use Date::Reformat;
 
-    my $parser = Date::Reformat->new(
+    my $reformat = Date::Reformat->new(
         parser => {
             regex  => qr/^(\d{4})-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)$/,
             params => [qw(year month day hour minute second)],
@@ -29,7 +29,7 @@ Date::Reformat - Rearrange date strings
         },
     );
 
-    my $parser = Date::Reformat->new(
+    my $reformat = Date::Reformat->new(
         parser => {
             strptime => '%Y-%m-%dT%M:%H:%S',
             # or heuristic => 'ymd', # http://www.postgresql.org/docs/9.2/static/datetime-input-rules.html
@@ -45,7 +45,7 @@ Date::Reformat - Rearrange date strings
         },
     );
 
-    my $reformatted_string = $parser->reformat_date($date_string);
+    my $reformatted_string = $reformat->reformat_date($date_string);
 
 =head1 DESCRIPTION
 
@@ -53,13 +53,24 @@ This module aims to be a lightweight and flexible tool for rearranging
 components of a date string, then returning the components in the order
 and structure specified.
 
+My motivation was a month of trying to compare data from spreadsheets from
+several sources, and every single one used a different date format, which
+made comparison difficult.
+
+There are so many modules for doing date math, or parsing a specific date
+format.  I needed something that could take in pretty much any format
+and turn it into a single format that I could then use for comparison.
+
 =cut
 
 use 5.010000;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+use Types::Standard qw(ClassName Object Maybe Optional slurpy Dict HashRef ArrayRef RegexpRef CodeRef Enum Str Int);
+use Type::Params qw();
+
+our $VERSION = '0.03';
 
 my $MONTH_LOOKUP = {
 };
@@ -338,44 +349,188 @@ my $DEFAULT_TRANSFORMATIONS = {
 
 =item new()
 
+Returns a new reformatter instance.
+
+    my $reformat = Date::Reformat->new(
+        'parser'          => $parsing_instructions,
+        'transformations' => $transformation_instructions,
+        'defaults'        => $default_values,
+        'formatter'       => $formatting_instructions,
+        'debug'           => 0,
+    );
+
+Parameters:
+
+=over 4
+
+=item parser
+
+A hashref of instructions used to initialize a parser.
+
+See L</"prepare_parser()"> for details.
+
+=item transformations
+
+An arrayref of hashrefs containing instructions on how to
+convert values of one token into values for another token
+(such as C<month_abbr> to C<month>).
+
+See L</"prepare_transformations()"> for details.
+
+=item defaults
+
+A hashref specifying values to use if the date string does
+not contain a specific token (such as a time_zone value).
+
+See L</"prepare_defaults()"> for details.
+
+=item formatter
+
+A hashref of instructions used to initialize a formatter.
+
+See L</"prepare_formatter()"> for details.
+
+=item debug
+
+Either a 1 or a 0, to turn debugging on or off, respectively.
+
+=back
+
 =cut
 
 sub new {
-    my ($class, %args) = @_;
+    state $check = Type::Params::compile(
+        ClassName,
+        slurpy Dict[
+            'debug'           => Optional[Int],
+            'parser'          => Optional[HashRef],
+            'formatter'       => Optional[HashRef],
+            'transformations' => Optional[ArrayRef[HashRef]],
+            'defaults'        => Optional[HashRef],
+        ],
+    );
+    my ($class, $args) = $check->(@_);
     my $self = bless {}, $class;
+
+    $self->debug($args->{'debug'});
+
     foreach my $parameter (
-        'debug',
         'parser',
         'formatter',
         'transformations',
         'defaults',
     )
     {
-        my $method = 'initialize_' . $parameter;
-        $self->$method($args{$parameter});
+        next if ! defined $args->{$parameter};
+
+        my $initialize = 'prepare_' . $parameter;
+        my @data = $self->$initialize($args->{$parameter});
+
+        my $add = 'add_' . $parameter;
+        $self->$add(@data);
     }
     return $self;
 }
 
-=item initialize_parser()
+=item prepare_parser()
+
+Builds a parser based on the given instructions.  To add it to
+the currently active parsers, see L</"add_parser()">.
+
+If several parsers are active, the first one to successfully parse
+the current date string returns the results of the parse, and subsequent
+parsers are not utilized.  See L</"parse_date()"> for more information.
+
+The types of parsers that can be initialized via this method are:
+
+=over 4
+
+=item regex
+
+The regex must specify what parts should be captured, and a list
+of token names must be supplied to identify which token each captured
+value will be assigned to.
+
+    $reformat->prepare_parser(
+        {
+            regex  => qr/^(\d{4})-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)$/,
+            params => [qw(year month day hour minute second)],
+        },
+    );
+
+=item regex with named capture
+
+The regex must specify what parts should be captured, using named
+capture syntax.
+
+    $reformat->prepare_parser(
+        {
+            regex  => qr/^(?<year>\d{4})-(?<month>\d\d)-(?<day>\d\d) (?<hour>\d\d?):(?<minute>\d\d):(?<second>\d\d)$/,
+        },
+    );
+
+=item strptime
+
+The format string must be in strptime() format.
+
+    $reformat->prepare_parser(
+        {
+            strptime => '%Y-%m-%dT%M:%H:%S',
+        },
+    );
+
+=item heuristic
+
+A hint must be provided that will help the parser determine the meaning
+of numbers if the ordering is ambiguous.
+
+Currently the heuristic parsing mimics the PostgreSQL date parser (though
+I have not copied over all the test cases from the PostgreSQL regression
+tests, so there are likely to be differences/flaws).
+
+    $reformat->prepare_parser(
+        {
+            heuristic => 'ymd',  # or 'mdy' or 'dmy'
+        },
+    );
+
+Currently when the heuristic parser parses a date string, it creates a
+named regex parser which it injects into the active parsers directly in
+front of itself, so that subsequent date strings that are in the same
+format will be parsed via the regex.
+
+I plan to add a parameter that will control whether parsers are generated
+by the heuristic parser (I also plan to refactor that method quite a bit,
+because it kind of makes me cringe to look at it).
+
+=back
 
 =cut
 
-sub initialize_parser {
-    my ($self, $definition) = @_;
-    # TODO: Verify $definition is a hashref with one of the approved parser parameters (regex, strptime, etc.).
+sub prepare_parser {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'regex'     => Optional[RegexpRef],
+            'params'    => Optional[ArrayRef],
+            'strptime'  => Optional[Str],
+            'heuristic' => Optional[Enum[qw(ymd dmy mdy)]],
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
+
     if (defined($definition->{'regex'})) {
 
         # Initialize the right kind of regex parser (simple capture or named capture).
         if (defined($definition->{'params'})) {
-            return $self->initialize_parser_for_regex_with_params(
+            return $self->prepare_parser_for_regex_with_params(
                 {
                     'regex'  => $definition->{'regex'},
                     'params' => $definition->{'params'},
                 }
             );
         }
-        return $self->initialize_parser_for_regex_named_capture(
+        return $self->prepare_parser_for_regex_named_capture(
             {
                 'regex' => $definition->{'regex'},
             },
@@ -384,7 +539,7 @@ sub initialize_parser {
     }
 
     if (defined($definition->{'strptime'})) {
-        return $self->initialize_parser_for_strptime(
+        return $self->prepare_parser_for_strptime(
             {
                 'strptime' => $definition->{'strptime'},
             },
@@ -392,7 +547,7 @@ sub initialize_parser {
     }
 
     if (defined($definition->{'heuristic'})) {
-        return $self->initialize_parser_heuristic(
+        return $self->prepare_parser_heuristic(
             {
                 'heuristic' => $definition->{'heuristic'},
             },
@@ -403,15 +558,98 @@ sub initialize_parser {
     return;
 }
 
-=item initialize_formatter()
+=item prepare_formatter()
+
+Builds a formatter based on the given instructions.  To add it to the
+currently active formatters, see L</"add_formatter">.
+
+If several formatters are active, they are each called in turn, receiving
+the output from the previous parser.
+
+The types of parsers that can be initialized via this method are:
+
+=over 4
+
+=item sprintf
+
+The format string must be in sprintf() format, and a list of token names
+must be supplied to identify which token values to send to the formatter.
+
+    $reformat->prepare_formatter(
+        {
+            sprintf => '%s-%02d-%02dT%02d:%02d:02d %s',
+            params  => [qw(year month day hour minute second time_zone)],
+        },
+    );
+
+=item strftime
+
+The format string must be in strftime() format.
+
+    $reformat->prepare_formatter(
+        {
+            strftime => '%Y-%m-%dT%M:%H:%S %Z',
+        },
+    );
+
+=item data_structure
+
+The type of the desired data structure must be specified, and a list of
+token names to identify which token values to include in the data structure.
+
+Valid data structure types are:
+
+=over 4
+
+=item hash
+
+=item hashref
+
+=item array
+
+=item arrayref
+
+=back
+
+    $reformat->prepare_formatter(
+        {
+            data_structure => 'hashref',
+            params         => [qw(year month day hour minute second time_zone)],
+        },
+    );
+
+=item coderef
+
+The supplied coderef will be passed the token values specified.  Whatever the
+coderef returns will be passed to the next active formatter, or will be returned,
+if this is the final formatter.
+
+    $reformat->prepare_formatter(
+        {
+            coderef => sub { my ($y, $m, $d) = @_; DateTime->new(year => $y, month => $m, day => $d) },
+            params  => [qw(year month day)],
+        },
+    );
+
+=back
 
 =cut
 
-sub initialize_formatter {
-    my ($self, $definition) = @_;
-    # TODO: Verify $definition is a hashref with one of the approved formatter parameters (sprintf, strftime, etc.).
+sub prepare_formatter {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'sprintf'        => Optional[Str],
+            'params'         => Optional[ArrayRef],
+            'strftime'       => Optional[Str],
+            'data_structure' => Optional[Enum[qw(hash hashref array arrayref)]],
+            'coderef'        => Optional[CodeRef],
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
+
     if (defined($definition->{'sprintf'})) {
-        return $self->initialize_formatter_for_sprintf(
+        return $self->prepare_formatter_for_sprintf(
             {
                 'sprintf' => $definition->{'sprintf'},
                 'params'  => $definition->{'params'},
@@ -420,27 +658,27 @@ sub initialize_formatter {
     }
 
     if (defined($definition->{'strftime'})) {
-        return $self->initialize_formatter_for_strftime(
+        return $self->prepare_formatter_for_strftime(
             {
                 'strftime' => $definition->{'strftime'},
             },
         );
     }
 
-    if (defined($definition->{'structure'})) {
-        if ($definition->{'structure'} =~ /^hash(?:ref)?$/) {
-            return $self->initialize_formatter_for_hashref(
+    if (defined($definition->{'data_structure'})) {
+        if ($definition->{'data_structure'} =~ /^hash(?:ref)?$/) {
+            return $self->prepare_formatter_for_hashref(
                 {
-                    'structure' => $definition->{'structure'},
+                    'structure' => $definition->{'data_structure'},
                     'params'    => $definition->{'params'},
                 },
             );
         }
 
-        if ($definition->{'structure'} =~ /^array(?:ref)?$/) {
-            return $self->initialize_formatter_for_arrayref(
+        if ($definition->{'data_structure'} =~ /^array(?:ref)?$/) {
+            return $self->prepare_formatter_for_arrayref(
                 {
-                    'structure' => $definition->{'structure'},
+                    'structure' => $definition->{'data_structure'},
                     'params'    => $definition->{'params'},
                 },
             );
@@ -448,7 +686,7 @@ sub initialize_formatter {
     }
 
     if (defined($definition->{'coderef'})) {
-        return $self->initialize_formatter_for_coderef(
+        return $self->prepare_formatter_for_coderef(
             {
                 'coderef' => $definition->{'coderef'},
                 'params'  => $definition->{'params'},
@@ -460,13 +698,91 @@ sub initialize_formatter {
     return;
 }
 
-=item initialize_transformations()
+=item prepare_transformations()
+
+Accepts an arrayref of hashrefs that specify how to transform
+token values from one token type to another.
+
+Returns the same arrayref.  To add it to the currently active
+transformers, see L</"add_transformations">.
 
 =cut
 
-sub initialize_transformations {
+sub prepare_transformations {
     my ($self, $transformations) = @_;
-    # TODO: Verify $transformations is an arrayref.
+    return $transformations // [];
+}
+
+=item add_transformations()
+
+Accepts an arrayref of hashrefs that specify how to transform
+token values from one token type to another.  Adds each
+transformation instruction to the list of active transformers.
+A transformation instruction with the same C<to> and C<from>
+values as a previous instruction will overwrite the previous
+version.
+
+    $reformat->add_transformations(
+        [
+            {
+                'to'             => 'hour',
+                'from'           => 'hour_12',
+                'transformation' => sub {
+                    my ($date) = @_;
+                    # Use the value of $date->{'hour_12'} (and $date->{'am_or_pm'})
+                    # to calculate what the value of $date->{'hour'} should be.
+                    # ...
+                    return $hour;
+                },
+            },
+        ],
+    );
+
+The values in each hashref are:
+
+=over 4
+
+=item to
+
+The name of the token type that is desired (for instance
+'hour', meaning the 24-hour format).
+
+=item from
+
+The name of the token type that is available in the date
+string (for instance 'hour_12', meaning the 12-hour format).
+
+=item transformation
+
+A coderef which accepts a hashref containing the information
+which has been parsed out of the date string.  The coderef
+is expected to examine the date information, transform the
+token type specified via C<from> into the correct value for the
+token type specified via C<to>, and return that value.
+
+=back
+
+Several transformations have been built into this module.
+Search for C<$DEFAULT_TRANSFORMATIONS> in the source code.
+
+Transformations added via this method will take precedence
+over built-in transformations.
+
+=cut
+
+sub add_transformations {
+    state $check = Type::Params::compile(
+        Object,
+        ArrayRef[
+            Dict[
+                'to'             => Str,
+                'from'           => Str,
+                'transformation' => CodeRef,
+            ],
+        ],
+    );
+    my ($self, $transformations) = $check->(@_);
+
     my $count = 0;
     foreach my $t (@$transformations) {
         $self->{'transformations'}->{$t->{'to'}}->{$t->{'from'}} = $t->{'transformation'};
@@ -475,36 +791,103 @@ sub initialize_transformations {
     return $count;
 }
 
-=item initialize_defaults()
+=item prepare_defaults()
+
+Accepts a hashref of default values to use when transforming
+or formatting a date which is missing tokens that are needed.
+
+This method clears out any defaults which had been set
+previously.
+
+Returns the same hashref it was given, but does not set them.
+To add defaults, see L</"add_defaults">.
 
 =cut
 
-sub initialize_defaults {
+sub prepare_defaults {
     my ($self, $args) = @_;
-    # TODO: Verify $args is a hashref.
-    return $self->{'defaults'} = $args // {};
+    $self->{'defaults'} = {};
+    return $args;
 }
 
-=item initialize_debug()
+=item add_defaults()
+
+Accepts a hashref of default values to use when transforming
+or formatting a date which is missing tokens that are needed.
+
+Each key should be the name of a token, and the corresponding
+value is the default value that will be used when a date is
+missing that token.
+
+    $reformat->add_defaults(
+        {
+            'time_zone' => 'America/New_York',
+        },
+    );
 
 =cut
 
-sub initialize_debug {
-    my ($self, $value) = @_;
-    return $self->{'debug'} = $value // 0;
+sub add_defaults {
+    state $check = Type::Params::compile(
+        Object,
+        HashRef,
+    );
+    my ($self, $args) = $check->(@_);
+
+    foreach my $token (keys %$args) {
+        $self->{'defaults'}->{$token} = $args->{$token};
+    }
+    return $self->{'defaults'};
 }
 
-=item initialize_parser_for_regex_with_params()
+=item debug()
+
+Turns debugging statements on or off, or returns the
+current debug setting.
+
+Expects a true value to turn debugging on, and a false value
+to turn debugging off.
+
+    $reformat->debug(1);  # 1 or 0
 
 =cut
 
-sub initialize_parser_for_regex_with_params {
-    my ($self, $definition) = @_;
+sub debug {
+    state $check = Type::Params::compile(
+        Object,
+        Maybe[Int],
+    );
+    my ($self, $value) = $check->(@_);
+    $self->{'debug'} = $value if (defined $value);
+    return $self->{'debug'} //= 0;
+}
+
+=item prepare_parser_for_regex_with_params()
+
+Internal method called by L</"prepare_parser()">.
+
+=cut
+
+sub prepare_parser_for_regex_with_params {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'regex'  => RegexpRef,
+            'params' => ArrayRef,
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
+
     my $regex = $definition->{'regex'};
     my $params = $definition->{'params'};
-    my $success = $self->add_parser(
+
+    state $sub_check = Type::Params::compile(
+        Str,
+    );
+
+    return (
         sub {
-            my ($date_string) = @_;
+            my ($date_string) = $sub_check->(@_);
             my (@components) = $date_string =~ $regex;
             return if ! @components;
             my %date = ();
@@ -513,19 +896,32 @@ sub initialize_parser_for_regex_with_params {
             return \%date;
         },
     );
-    return $success;
 }
 
-=item initialize_parser_for_regex_named_capture()
+=item prepare_parser_for_regex_named_capture()
+
+Internal method called by L</"prepare_parser()">.
 
 =cut
 
-sub initialize_parser_for_regex_named_capture {
-    my ($self, $definition) = @_;
+sub prepare_parser_for_regex_named_capture {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'regex' => RegexpRef,
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
+
     my $regex = $definition->{'regex'};
-    my $success = $self->add_parser(
+
+    state $sub_check = Type::Params::compile(
+        Str,
+    );
+
+    return (
         sub {
-            my ($date_string) = @_;
+            my ($date_string) = $sub_check->(@_);
             my $success = $date_string =~ $regex;
             return if ! $success;
             my %date = %+;
@@ -546,15 +942,23 @@ sub initialize_parser_for_regex_named_capture {
             return \%date;
         },
     );
-    return $success;
 }
 
-=item initialize_parser_for_strptime()
+=item prepare_parser_for_strptime()
+
+Internal method called by L</"prepare_parser()">.
 
 =cut
 
-sub initialize_parser_for_strptime {
-    my ($self, $definition) = @_;
+sub prepare_parser_for_strptime {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'strptime' => Str,
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
+
     my $strptime = $definition->{'strptime'};
     my $format = $strptime;
 
@@ -593,20 +997,28 @@ sub initialize_parser_for_strptime {
     }
 
     say "Crafted regex: $strptime -> $format" if $self->{'debug'};
-    my $success = $self->initialize_parser_for_regex_named_capture(
+    return $self->prepare_parser_for_regex_named_capture(
         {
             'regex' => qr/$format/,
         },
     );
-    return $success;
 }
 
-=item initialize_parser_heuristic()
+=item prepare_parser_heuristic()
+
+Internal method called by L</"prepare_parser()">.
 
 =cut
 
-sub initialize_parser_heuristic {
-    my ($self, $definition) = @_;
+sub prepare_parser_heuristic {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'heuristic' => Enum[qw(ymd dmy mdy)],
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
+
     my $hint = $definition->{'heuristic'};
     my $known_parsers = {}; # Populated when we add a parser to the stack in front of this one.
     my $regex_for_date = qr{ \w+ [-/\.] \w+ (?:[-/\.] \w+) }x;
@@ -637,9 +1049,14 @@ sub initialize_parser_heuristic {
         # anything else
         | ( . )
     }x;
-    my $success = $self->add_parser(
+
+    state $sub_check = Type::Params::compile(
+        Str,
+    );
+
+    return (
         sub {
-            my ($date_string) = @_;
+            my ($date_string) = $sub_check->(@_);
             my $order_string; # Will be set with ymd|dmy|mdy when we have enough information.
 
             # Split string into parts that can be identified later.
@@ -1008,10 +1425,12 @@ sub initialize_parser_heuristic {
             # Add a new parser that will match this date format.
             if (! defined($known_parsers->{$parser_regex}) ) {
                 $known_parsers->{$parser_regex} = 1;
-                $self->initialize_parser_for_regex_named_capture(
-                    {
-                        'regex' => qr/$parser_regex/,
-                    },
+                $self->add_parser(
+                    $self->prepare_parser_for_regex_named_capture(
+                        {
+                            'regex' => qr/$parser_regex/,
+                        },
+                    ),
                 );
                 # Move the heuristic parser to the last slot again.
                 push(
@@ -1025,21 +1444,34 @@ sub initialize_parser_heuristic {
             return $date;
         },
     );
-    return $success;
 }
 
-=item initialize_formatter_for_arrayref()
+=item prepare_formatter_for_arrayref()
+
+Internal method called by L</"prepare_formatter()">.
 
 =cut
 
-sub initialize_formatter_for_arrayref {
-    my ($self, $definition) = @_;
+sub prepare_formatter_for_arrayref {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'params'    => ArrayRef[Str],
+            'structure' => Optional[Enum[qw(array arrayref)]],
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
+
     my $structure = $definition->{'structure'} // 'arrayref';
-    my $params = $definition->{'params'} // die "Unable to create $structure formatter: No 'params' argument defined.";
-    # TODO: Validate parameters.
-    my $success = $self->add_formatter(
+    my $params = $definition->{'params'};
+
+    state $sub_check = Type::Params::compile(
+        HashRef,
+    );
+
+    return (
         sub {
-            my ($date) = @_;
+            my ($date) = $sub_check->(@_);
             my @formatted = (
                 map
                 {
@@ -1064,73 +1496,108 @@ sub initialize_formatter_for_arrayref {
             return @formatted;
         },
     );
-    return $success;
 }
 
-=item initialize_formatter_for_hashref()
+=item prepare_formatter_for_hashref()
+
+Internal method called by L</"prepare_formatter()">.
 
 =cut
 
-sub initialize_formatter_for_hashref {
-    my ($self, $definition) = @_;
-    my $structure = $definition->{'structure'} // 'hashref';
-    my $params = $definition->{'params'} // die "Unable to create $structure formatter: No 'params' argument defined.";
-    # TODO: Validate parameters.
+sub prepare_formatter_for_hashref {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'params'    => ArrayRef[Str],
+            'structure' => Optional[Enum[qw(hash hashref)]],
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
 
-    $self->initialize_formatter_for_arrayref(
+    my $structure = $definition->{'structure'} // 'hashref';
+    my $params = $definition->{'params'};
+
+    my @formatters = $self->prepare_formatter_for_arrayref(
         {
             'structure' => 'arrayref',
             'params'    => $params,
         },
     );
 
-    my $success = $self->add_formatter(
+    state $sub_check = Type::Params::compile(
+        ArrayRef,
+    );
+
+    push @formatters, (
         sub {
-            my ($date) = @_;
+            my ($date) = $sub_check->(@_);
             my %formatted = ();
             @formatted{@$params} = @$date;
             return \%formatted if $structure eq 'hashref';
             return %formatted;
         },
     );
-    return $success;
+    return @formatters;
 }
 
-=item initialize_formatter_for_coderef()
+=item prepare_formatter_for_coderef()
+
+Internal method called by L</"prepare_formatter()">.
 
 =cut
 
-sub initialize_formatter_for_coderef {
-    my ($self, $definition) = @_;
-    my $coderef = $definition->{'coderef'} // sub { @_ };
-    my $params = $definition->{'params'} // die "Unable to create coderef formatter: No 'params' argument defined.";
-    # TODO: Validate parameters.
+sub prepare_formatter_for_coderef {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'params'  => ArrayRef[Str],
+            'coderef' => CodeRef,
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
 
-    $self->initialize_formatter_for_arrayref(
+    my $coderef = $definition->{'coderef'};
+    my $params = $definition->{'params'};
+
+    my @formatters = $self->prepare_formatter_for_arrayref(
         {
             'structure' => 'array',
             'params'    => $params,
         },
     );
 
-    my $success = $self->add_formatter(
+    push @formatters, (
         $coderef,
     );
-    return $success;
+    return @formatters;
 }
 
-=item initialize_formatter_for_sprintf()
+=item prepare_formatter_for_sprintf()
+
+Internal method called by L</"prepare_formatter()">.
 
 =cut
 
-sub initialize_formatter_for_sprintf {
-    my ($self, $definition) = @_;
+sub prepare_formatter_for_sprintf {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'params'  => ArrayRef[Str],
+            'sprintf' => Str,
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
+
     my $sprintf = $definition->{'sprintf'};
-    my $params = $definition->{'params'} // die "Unable to create sprintf formatter: No 'params' argument defined.";
-    # TODO: Validate parameters.
-    my $success = $self->add_formatter(
+    my $params = $definition->{'params'};
+
+    state $sub_check = Type::Params::compile(
+        HashRef,
+    );
+
+    return (
         sub {
-            my ($date) = @_;
+            my ($date) = $sub_check->(@_);
             my $formatted = sprintf(
                 $sprintf,
                 map
@@ -1155,15 +1622,23 @@ sub initialize_formatter_for_sprintf {
             return $formatted;
         },
     );
-    return $success;
 }
 
-=item initialize_formatter_for_strftime()
+=item prepare_formatter_for_strftime()
+
+Internal method called by L</"prepare_formatter()">.
 
 =cut
 
-sub initialize_formatter_for_strftime {
-    my ($self, $definition) = @_;
+sub prepare_formatter_for_strftime {
+    state $check = Type::Params::compile(
+        Object,
+        Dict[
+            'strftime' => Str,
+        ],
+    );
+    my ($self, $definition) = $check->(@_);
+
     my $strftime = $definition->{'strftime'};
     my $format = $strftime;
     my $params = [];
@@ -1208,21 +1683,27 @@ sub initialize_formatter_for_strftime {
     }
 
     say "Crafted sprintf: $strftime -> $format [" . join(', ', @$params) . "]" if $self->{'debug'};
-    my $success = $self->initialize_formatter_for_sprintf(
+    return $self->prepare_formatter_for_sprintf(
         {
             'sprintf' => $format,
             'params'  => $params,
         },
     );
-    return $success;
 }
 
 =item strptime_token_to_regex()
 
+Internal method called by L</"prepare_parser()">.
+
 =cut
 
 sub strptime_token_to_regex {
-    my ($self, $token) = @_;
+    state $check = Type::Params::compile(
+        Object,
+        Str,
+    );
+    my ($self, $token) = $check->(@_);
+
     my $internal;
     say "Attempting to convert strptime token $token into a regex." if $self->{'debug'};
     if (defined($self->{'strptime_mappings'}->{$token})) {
@@ -1247,10 +1728,17 @@ sub strptime_token_to_regex {
 
 =item strftime_token_to_internal
 
+Internal method called by L</"prepare_formatter()">.
+
 =cut
 
 sub strftime_token_to_internal {
-    my ($self, $token) = @_;
+    state $check = Type::Params::compile(
+        Object,
+        Str,
+    );
+    my ($self, $token) = $check->(@_);
+
     my $internal;
     say "Attempting to convert strftime token $token into an internal token." if $self->{'debug'};
     if (defined($self->{'strftime_mappings'}->{$token})) {
@@ -1281,12 +1769,22 @@ sub strftime_token_to_internal {
 
 =item transform_token_value()
 
+Internal method called by L</"prepare_formatter()">.
+
 =cut
 
 sub transform_token_value {
-    my ($self, %args) = @_;
-    my $target_token = $args{'target_token'};
-    my $date = $args{'date'};
+    state $check = Type::Params::compile(
+        Object,
+        slurpy Dict[
+            'target_token' => Str,
+            'date'         => HashRef,
+        ],
+    );
+    my ($self, $args) = $check->(@_);
+
+    my $target_token = $args->{'target_token'};
+    my $date = $args->{'date'};
 
     # Return the value, if it is already set.
     return $date->{$target_token} if defined($date->{$target_token});
@@ -1308,15 +1806,28 @@ sub transform_token_value {
 
 =item most_likely_token()
 
+Internal method called by L</"prepare_parser()">.
+
 =cut
 
 sub most_likely_token {
-    my ($self, %args) = @_;
-    my $already_claimed = $args{'already_claimed'} // {};
-    my $possible_tokens = $args{'possible_tokens'} // return;
-    my $hint = $args{'heuristic'} // '';
-    my $date_part = $args{'value'} // return;
-    my $date_string = $args{'date_string'} // $date_part;
+    state $check = Type::Params::compile(
+        Object,
+        slurpy Dict[
+            'already_claimed' => Optional[HashRef],
+            'possible_tokens' => ArrayRef,
+            'heuristic'       => Optional[Str],
+            'value'           => Str,
+            'date_string'     => Optional[Str],
+        ],
+    );
+    my ($self, $args) = $check->(@_);
+
+    my $already_claimed = $args->{'already_claimed'} // {};
+    my $possible_tokens = $args->{'possible_tokens'};
+    my $hint = $args->{'heuristic'} // '';
+    my $date_part = $args->{'value'};
+    my $date_string = $args->{'date_string'} // $date_part;
 
     foreach my $token (@$possible_tokens) {
         if ($token eq 'day') {
@@ -1368,33 +1879,80 @@ sub most_likely_token {
 
 =item add_parser()
 
+Adds a parser to the active parsers.  When parsing a date string,
+the parser will be called if each preceeding parser has failed to
+parse the date.
+
+See L</"prepare_parser()"> for generating a parser in the correct
+format.
+
+    $reformat->add_parser(
+        $reformat->prepare_parser( ... ),
+    );
+
 =cut
 
 sub add_parser {
-    my ($self, $parser) = @_;
-    my $count = push @{ $self->{'active_parsers'} }, $parser;
+    state $check = Type::Params::compile(
+        Object,
+        slurpy ArrayRef[CodeRef],
+    );
+    my ($self, $parsers) = $check->(@_);
+
+    my $count = push @{ $self->{'active_parsers'} }, @$parsers;
     return $count ? 1 : 0;
 }
 
 =item add_formatter()
 
+Adds a formatter to the active formatters.  When formatting a date,
+the formatter will be called after each preceeding formatter, receiving
+as input the output from the previous formatter.
+
+See L</"prepare_formatter()"> for generating a formatter in the correct
+format.
+
+    $reformat->add_formatter(
+        $reformat->prepare_formatter( ... ),
+    );
+
 =cut
 
 sub add_formatter {
-    my ($self, $formatter) = @_;
-    my $count = push @{ $self->{'active_formatters'} }, $formatter;
+    state $check = Type::Params::compile(
+        Object,
+        slurpy ArrayRef[CodeRef],
+    );
+    my ($self, $formatters) = $check->(@_);
+
+    my $count = push @{ $self->{'active_formatters'} }, @$formatters;
     return $count ? 1 : 0;
 }
 
 =item parse_date()
 
+Given a date string, attempts to parse it via the active parsers.
+Returns a hashref containing the tokens that were extracted
+from the date string.
+
+    my $date_hashref = $reformat->parse_date($date_string);
+
 =cut
 
 sub parse_date {
-    my ($self, $date_string) = @_;
+    state $check = Type::Params::compile(
+        Object,
+        Str,
+    );
+    my ($self, $date_string) = $check->(@_);
+
+    state $has_parser = ArrayRef[CodeRef];
+    if (! $has_parser->($self->{'active_parsers'}) ) {
+        die "No parsers defined. Have you called add_parser()?";
+    }
+
     foreach my $parser (@{ $self->{'active_parsers'} }) {
         my $date = $parser->($date_string);
-        # TODO: Add formatting step here.
         return $date if defined($date);
     }
     # None of the parsers were able to extract the date components.
@@ -1403,10 +1961,28 @@ sub parse_date {
 
 =item format_date()
 
+Given a hashref containing the tokens that were extracted from a
+date string, formats the date using each of the active parsers,
+passing the output from the previous formatter to the next formatter.
+
+    my $date_string = $reformat->format_date($date_hashref);
+
 =cut
 
 sub format_date {
-    my ($self, $date) = @_;
+    state $check = Type::Params::compile(
+        Object,
+        Maybe[HashRef],
+    );
+    my ($self, $date) = $check->(@_);
+
+    return if ! defined($date);
+
+    state $has_formatter = ArrayRef[CodeRef];
+    if (! $has_formatter->($self->{'active_formatters'}) ) {
+        die "No formatters defined. Have you called add_formatter()?";
+    }
+
     my @formatted = ($date);
     foreach my $formatter (@{ $self->{'active_formatters'} }) {
         @formatted = $formatter->(@formatted);
@@ -1417,10 +1993,20 @@ sub format_date {
 
 =item reformat_date()
 
+Given a date string, attempts to parse it and format it using the
+active parsers and formaters.
+
+    my $date_string = $reformat->reformat_date($date_string);
+
 =cut
 
 sub reformat_date {
-    my ($self, $date_string) = @_;
+    state $check = Type::Params::compile(
+        Object,
+        Str,
+    );
+    my ($self, $date_string) = $check->(@_);
+
     my $date = $self->parse_date($date_string);
     my @formatted = $self->format_date($date);
     return $formatted[0] if (scalar(@formatted) == 1);
@@ -1439,7 +2025,7 @@ __END__
 
 =item Date::Transform
 
-=item Date::Parser
+=item Date::Parse
 
 =item Date::Format
 
